package com.ruoyi.common.code;


import cn.hutool.core.util.StrUtil;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.imageio.ImageIO;
import javax.swing.filechooser.FileSystemView;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * 二维码工具
 **/
@Component
public class QRCodeUtils {
    private static final Logger log= LoggerFactory.getLogger(QRCodeUtils.class);

    //CODE_WIDTH：二维码宽度，单位像素
    private static final int CODE_WIDTH = 400;
    //CODE_HEIGHT：二维码高度，单位像素
    private static final int CODE_HEIGHT = 400;
    //FRONT_COLOR：二维码前景色，0x000000 表示黑色
    private static final int FRONT_COLOR = 0x000000;
    //BACKGROUND_COLOR：二维码背景色，0xFFFFFF 表示白色
    //演示用 16 进制表示，和前端页面 CSS 的取色是一样的，注意前后景颜色应该对比明显，如常见的黑白
    private static final int BACKGROUND_COLOR = 0xFFFFFF;

    public static void createCodeToFile(QrDTO content, File codeImgFileSaveDir, String fileName) {
        try {
            if (content == null || StrUtil.isBlank(content.getValue()) || StrUtil.isBlank(fileName)) {
                return;
            }
            if (codeImgFileSaveDir==null || codeImgFileSaveDir.isFile()) {
                //二维码图片存在目录为空，默认放在桌面...
                codeImgFileSaveDir = FileSystemView.getFileSystemView().getHomeDirectory();
            }
            if (!codeImgFileSaveDir.exists()) {
                //二维码图片存在目录不存在，开始创建...
                codeImgFileSaveDir.mkdirs();
            }

            //核心代码-生成二维码
            BufferedImage bufferedImage = getBufferedImage(content);

            File codeImgFile = new File(codeImgFileSaveDir, fileName);
            ImageIO.write(bufferedImage, "png", codeImgFile);

            log.info("二维码图片生成成功：" + codeImgFile.getPath());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 生成二维码并输出到输出流, 通常用于输出到网页上进行显示，输出到网页与输出到磁盘上的文件中，区别在于最后一句 ImageIO.write
     * write(RenderedImage im,String formatName,File output)：写到文件中
     * write(RenderedImage im,String formatName,OutputStream output)：输出到输出流中
     * @param content  ：二维码内容
     * @param outputStream ：输出流，比如 HttpServletResponse 的 getOutputStream
     */
    public static void createCodeToOutputStream(QrDTO content, OutputStream outputStream) {
        try {
            if (content == null || StrUtil.isBlank(content.getValue())) {
                return;
            }
            //核心代码-生成二维码
            BufferedImage bufferedImage = getBufferedImage(content);

            //区别就是这一句，输出到输出流中，如果第三个参数是 File，则输出到文件中
            ImageIO.write(bufferedImage, "png", outputStream);

            log.info("二维码图片生成到输出流成功...");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //核心代码-生成二维码
    private static BufferedImage getBufferedImage(QrDTO content) throws WriterException {

        String name = content.getName();
        String value = content.getValue().trim();

        //com.google.zxing.EncodeHintType：编码提示类型,枚举类型
        Map<EncodeHintType, Object> hints = new HashMap();

        //EncodeHintType.CHARACTER_SET：设置字符编码类型
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");

        //EncodeHintType.ERROR_CORRECTION：设置误差校正
        //ErrorCorrectionLevel：误差校正等级，L = ~7% correction、M = ~15% correction、Q = ~25% correction、H = ~30% correction
        //不设置时，默认为 L 等级，等级不一样，生成的图案不同，但扫描的结果是一样的
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.M);

        //EncodeHintType.MARGIN：设置二维码边距，单位像素，值越小，二维码距离四周越近
        hints.put(EncodeHintType.MARGIN, 1);

        /*
          MultiFormatWriter:多格式写入，这是一个工厂类，里面重载了两个 encode 方法，用于写入条形码或二维码
               encode(String contents,BarcodeFormat format,int width, int height,Map<EncodeHintType,?> hints)
               contents:条形码/二维码内容
               format：编码类型，如 条形码，二维码 等
               width：码的宽度
               height：码的高度
               hints：码内容的编码类型
          BarcodeFormat：枚举该程序包已知的条形码格式，即创建何种码，如 1 维的条形码，2 维的二维码 等
          BitMatrix：位(比特)矩阵或叫2D矩阵，也就是需要的二维码
         */
        // 生成二维码
        MultiFormatWriter multiFormatWriter = new MultiFormatWriter();
        /*参数顺序分别为：编码内容，编码类型，生成图片宽度，生成图片高度，设置参数
          BitMatrix 的 get(int x, int y) 获取比特矩阵内容，指定位置有值，则返回true，将其设置为前景色，否则设置为背景色
          BufferedImage 的 setRGB(int x, int y, int rgb) 方法设置图像像素
               x：像素位置的横坐标，即列
               y：像素位置的纵坐标，即行
               rgb：像素的值，采用 16 进制,如 0xFFFFFF 白色
         */
        BitMatrix bitMatrix = multiFormatWriter.encode(value, BarcodeFormat.QR_CODE, CODE_WIDTH, CODE_HEIGHT, hints);
        // 将二维码放入缓冲流
        BufferedImage bufferedImage = new BufferedImage(CODE_WIDTH, CODE_HEIGHT, BufferedImage.TYPE_INT_BGR);
        for (int x = 0; x < CODE_WIDTH; x++) {
            for (int y = 0; y < CODE_HEIGHT; y++) {
                // 循环将二维码内容写入图片
                bufferedImage.setRGB(x, y, bitMatrix.get(x, y) ? FRONT_COLOR : BACKGROUND_COLOR);
            }
        }

        // ------------------------------------------自定义文本描述-------------------------------------------------
        // 1、在内存创建图片缓冲区 这里设置画板的宽高和类型
        BufferedImage outImage = new BufferedImage(CODE_WIDTH, CODE_HEIGHT, BufferedImage.TYPE_4BYTE_ABGR);

        // 2、创建画布，获取图像对象
        Graphics2D graphics2D = outImage.createGraphics();

        // 3、抗锯齿，防止模糊
        RenderingHints rh = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        rh.put(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        rh.put(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
        graphics2D.setRenderingHints(rh);

        // 4、在画布上画上二维码 X轴Y轴，宽度高度
        graphics2D.drawImage(bufferedImage, 0, 0, bufferedImage.getWidth(), bufferedImage.getHeight(), null);

        // 设置文字颜色为黑色
        graphics2D.setColor(Color.BLACK);
        // 字体、字型、字号
        graphics2D.setFont(new Font("宋体", Font.PLAIN, 18));

        // 获取字体宽度
        int contentWidth = graphics2D.getFontMetrics().stringWidth(name);

        // drawString(文字信息、x轴、y轴)方法根据参数设置文字的坐标轴 ，根据需要来进行调整
        graphics2D.drawString(name, (CODE_WIDTH - contentWidth) / 2, CODE_HEIGHT - 5);

        graphics2D.dispose();
        outImage.flush();
        bufferedImage = outImage;

        return bufferedImage;
    }
}

